/************************************************************************************/
// Schalterfunktionalitt an einem Empfnger- RC- Ausgang
//
//  -> erprobt mit ATmega8
//
// Version: 0.1
//
// Datum: 26.12.2009
//
// Hardware bei ATtiny13 (Taktfrequenz (4,8Mhz, CKDIV8 Vorteiler Fuse gesetzt -> 600kHz):
// R/C-Kanal an PB1 (INT0)
// Error-LED an PB2
// Schalt-LED1 an PB3
// Schalt-LED2 an PB4

// Hardware bei ATmega8 (Taktfrequenz (1,0Mhz interner RC- Takt):
// R/C-Kanal an PD2 (INT0)
// Error-LED an PB2
// Schalt-LED1 an PB3
// Schalt-LED2 an PB4
/************************************************************************************/
// Compiler- Warnungen

#ifndef F_CPU
#warning "F_CPU not defined"
#define F_CPU 1000000UL
#endif

/************************************************************************************/
// Libraries

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>		// fr Watchdog- Funktionen
#include <stdint.h>

/************************************************************************************/
// Pin- Belegung

#define RC_KanalEingang		PIND	// Eingangssignal R/C- Empfnger
#define RC_KanalPin			PIND2	// Pin fr Eingangssignal R/C- Empfnger
#define RC_KanalPort		PORTD	// Port fr Eingangssignal R/C- Empfnger

#define LED_Port			PORTB	// Port fr Ausgnge
#define LED_PortDDR			DDRB	// Datenrichtungsregister fr Ausgnge
#define SchaltErrorLED		PB2		// Ausgang fr Error LED
#define SchaltLED1			PB3		// Ausgang fr Schalt LED 1
#define SchaltLED2			PB4		// Ausgang fr Schalt LED 2

/************************************************************************************/
// Variablen

static volatile uint8_t Reading;	// Bit- Merker zur Sperrung der Hauptroutine whrend der erneuten Wertermittlung
									// Merker Flanke

static volatile uint8_t Error;		// Bit- Merker fr Fehler
static volatile uint8_t RCvalue;	// empfangener Wert von R/C- Empfnger -> wird von Timer runtergezhlt

/************************************************************************************/
//Header

void RC_Read();
void RC_Error();

/************************************************************************************/
// Interruptroutinen

/* ISR fr INT0 - R/C-Signal mit Timer lesen */
ISR(INT0_vect)			  
{
	RC_Read();
}

/* Fehlerbehandlung bei Timerberlauf -> Fehler generieren */
ISR(TIMER0_OVF_vect)			  
{
	RC_Error();
}

/************************************************************************************/
// Hauptprogramm

int main(void)
{
// Vorbereitung des RC- Eingangs
	// RC- Eingang ist schon nach Initialisierung des AVR ein Eingang
	RC_KanalPort |= (1<<RC_KanalPin); // interne Pull-Up-Widerstnde aktivieren

// Initialisierung LED- Ausgnge
	LED_PortDDR |= (1<<SchaltErrorLED)|(1<<SchaltLED1)|(1<<SchaltLED2); // Datenrichtung
	LED_Port |= (1<<SchaltErrorLED)|(1<<SchaltLED1)|(1<<SchaltLED2);	// alle Portausgnge sind jetzt high
																		// -> LED sind nun aus

// Initialisierung Interrupteingang INT0
	MCUCR |= (1<<ISC00);	// Interrupt wird bei jedem Pegelwechsel an INT0 ausgelst
	GICR |= (1<<INT0);		// Interrupt INT0 aktivieren

// Initialisierung Timer0
	TIMSK |= (1<<TOIE0);	// Timer0 Interrupt aktivieren
	
// Initialisierung Watchdog auf 500ms, bedingt #include <avr/wdt.h>
	wdt_enable(WDTO_500MS);

// Vorbereitung Status- Flags - kein Fehler liegt an, momentan kein Datenenpfang
	Reading = 0;
	Error = 0;
	
// globale Interrupptfreigabe
	sei();

/***********************************************************************************/
	while(1)
	{
		// Watchdog zurcksetzen
		wdt_reset();

		// Error- LED an, wenn Timer- berlauf war
		if(Error == 1)
		{
			LED_Port &= ~(1<<SchaltErrorLED);	// Portausgang Error- LED low -> LED an
		}
		else
		{
			LED_Port |= (1<<SchaltErrorLED);	// Portausgang Error- LED high -> LED aus
		}
		
		// LED- Ausgabe whrend der langen Lesepause...
		if(Reading == 0)
		{
			// LED- Ausgnge schalten, je Timer- Wert -> erprobt mit ATmega8
			if(RCvalue > 170)
			{
				LED_Port &= ~(1<<SchaltLED1);	// Portausgang Schalt- LED2 low -> LED an

			}
			else
			{
				LED_Port |= (1<<SchaltLED1);	// Portausgang Schalt- LED1 high -> LED aus

			}
		
			if(RCvalue > 205)
			{
				LED_Port &= ~(1<<SchaltLED2);	// Portausgang Schalt- LED2 low -> LED an
			}
			else
			{
				LED_Port |= (1<<SchaltLED2);	// Portausgang Schalt- LED2 high -> LED aus
			}
		}	
	}
}

/***********************************************************************************/

// ISR fr INT0 - R/C-Signal mit Timer lesen
void RC_Read()
{
	// Timer starten mit steigender Flanke
	if(Reading == 0)
	{
		TCCR0 |= (1<<CS01);	// Start Timer0 mit Vorteiler 8
		Reading = 1;		// Merker Flanke setzten
	}	
	
	// Timer stoppen mit fallender Flanke
	else
	{
		TCCR0 = 0x00;		// Stop Timer0
		RCvalue = TCNT0;	// Wert von Timer lesen
		TCNT0 = 0x00;		// neuen Startwert fr Timer laden
		Reading = 0;		// Merker Flanke setzten
	}

	Error = 0;			// Error- Merker zurcksetzen
}


// ISR fr Timer0 - Fehlerhandling Timeroverflow
void RC_Error()
{
	TCCR0 = 0x00;			// Stop Timer0
	RCvalue = 0;			// Wert fr Ausgnge annehmen
	TCNT0 = 0x00;			// neuen Startwert fr Timer zurcksetzen
	Reading = 0;			// Merker Flanke setzten
	Error = 1;				// Error- Merker setzen
}

